﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using HarmonyLib;
using System.Reflection;
using UnityEngine.UIElements;

public class RemoveEvilAnimals : Mod
{
    #region Variables

    static bool removeBears;
    static bool beesDealsDamage;
    static bool removeButlerBots;
    static bool removeLukersAkaRats;
    static bool removePoisonPuffers;
    static bool removeScreechers;
    static bool removeSeagulls;
    static bool removeSharks;
    static bool removeWarthogs;
    static bool removeMudhogsAkaPigs;

    #endregion

    public void Start()
    {
        Debug.Log("Mod RemoveEvilAnimals has been loaded!");

        var harmony = new Harmony("com.KUNGERMOoN.RemoveEvilAnimals");
        harmony.PatchAll();
    }

    public void OnModUnload()
    {
        Debug.Log("Mod RemoveEvilAnimals has been unloaded!");
    }

    #region Patches

    [HarmonyPatch(typeof(AI_NetworkBehaviour), "Update")]
    class AnimalsPatch
    {
        static void Prefix(AI_NetworkBehaviour __instance)
        {
            if (!(__instance is AI_NetworkBehaviour_Domestic) & !(__instance is AI_NetworkBehaviour_BugSwarm))
            {
                if (__instance is AI_NetworkBehaviour_Bear && removeBears)
                    Destroy(__instance.gameObject);
                else if (__instance is AI_NetworkBehaviour_ButlerBot && removeButlerBots)
                    Destroy(__instance.gameObject);
                else if (__instance is AI_NetworkBehaviour_Rat && removeLukersAkaRats)
                    Destroy(__instance.gameObject);
                else if (__instance is AI_NetworkBehaviour_PufferFish && removePoisonPuffers)
                    Destroy(__instance.gameObject);
                else if (__instance is AI_NetworkBehaviour_StoneBird && removeScreechers)
                    Destroy(__instance.gameObject);
                else if (__instance is AI_NetworkBehavior_Shark && removeSharks)
                {
                    AI_NetworkBehavior_Shark shark = __instance as AI_NetworkBehavior_Shark;
                    Destroy(shark.stateMachineShark.gameObject);
                }
                else if (__instance is AI_NetworkBehaviour_Boar && removeWarthogs)
                    Destroy(__instance.gameObject);
                else if (__instance is AI_NetworkBehaviour_Pig && removeMudhogsAkaPigs)
                    Destroy(__instance.gameObject);
            }
        }
    }

    [HarmonyPatch(typeof(AI_StateMachine_Shark), "OnDestroy")]
    class SharkPatchOnDestroy
    {
        static bool Prefix(AI_StateMachine_Shark __instance)
        {
            if (removeSharks)
            {
                return false;
            }
            return true;
        }
    }

    [HarmonyPatch(typeof(AI_NetworkBehaviour_BugSwarm), "DamageClosestPlayer")]
    class BeesPatch
    {
        static bool Prefix()
        {
            if (beesDealsDamage)
            {
                return true;
            }
            return false;
        }
    }

    [HarmonyPatch(typeof(Seagull), "Update")]
    class SeagullsPatch
    {
        static void Prefix(Seagull __instance)
        {
            if (removeSeagulls)
                Destroy(__instance.gameObject);
        }
    }

    #endregion

    #region Settings
    //I used Extra Settings API (https://www.raftmodding.com/mods/extra-settings-api) for custom mod settings
    static HarmonyLib.Traverse ExtraSettingsAPI_Traverse;
    static bool ExtraSettingsAPI_Loaded = false;

    public void ExtraSettingsAPI_Load() // Occurs when the API loads the mod's settings
    {
        LoadSettings();
    }

    public void ExtraSettingsAPI_SettingsClose() // Occurs when user closes the settings menu
    {
        LoadSettings();
    }

    public void LoadSettings()
    {
        removeBears = ExtraSettingsAPI_GetCheckboxState("removeBears");
        beesDealsDamage = ExtraSettingsAPI_GetCheckboxState("beesDealsDamage");
        removeButlerBots = ExtraSettingsAPI_GetCheckboxState("removeButlerBots");
        removeLukersAkaRats = ExtraSettingsAPI_GetCheckboxState("removeLukersAkaRats");
        removePoisonPuffers = ExtraSettingsAPI_GetCheckboxState("removePoisonPuffers");
        removeScreechers = ExtraSettingsAPI_GetCheckboxState("removeScreechers");
        removeSeagulls = ExtraSettingsAPI_GetCheckboxState("removeSeagulls");
        removeSharks = ExtraSettingsAPI_GetCheckboxState("removeSharks");
        removeWarthogs = ExtraSettingsAPI_GetCheckboxState("removeWarthogs");
        removeMudhogsAkaPigs = ExtraSettingsAPI_GetCheckboxState("removeMudhogsAkaPigs");
    }

    public bool ExtraSettingsAPI_GetCheckboxState(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getCheckboxState", new object[] { this, SettingName }).GetValue<bool>();
        return false;
    }

    #endregion
}